import { world, system, GameMode, EquipmentSlot, Player } from "@minecraft/server";

const ITEM = 'drg:luminite_protector_item';
const ENTITY = 'drg:luminite_protector';
const PARTICLE = 'minecraft:knockback_roar_particle';
const SOUND = 'luminite_protector.summon';

world.afterEvents.itemUse.subscribe((event) => {
    const { itemStack, source } = event;

    if (itemStack.typeId == ITEM) {
        const cooldown = itemStack.getComponent('cooldown');

        if (cooldown.getCooldownTicksRemaining(source) == 0) {
            cooldown.startCooldown(source);

            const offsets = [
                { x: 1, z: 0 },
                { x: -1, z: 0 },
                { x: 0, z: 1 },
                { x: 0, z: -1 }
            ]

            const offset = offsets[Math.floor(Math.random() * offsets.length)];

            const newLoc = {
                x: source.location.x + offset.x,
                y: source.location.y,
                z: source.location.z + offset.z
            }

            world.getAllPlayers().forEach((player) => {
                const distance = getDistance(source.location, player.location);
                if (distance <= 16) {
                    player.playSound(SOUND);
                }
            })

            source.dimension.spawnParticle(PARTICLE, source.location);
            source.dimension.spawnEntity(ENTITY, newLoc);
        }
    }
})

function getDistance(pos1, pos2) {
    return Math.sqrt(Math.pow(pos1.x - pos2.x, 2) + Math.pow(pos1.y - pos2.y, 2) + Math.pow(pos1.z - pos2.z, 2));
}